import { z } from 'zod';
import { FIRESTORE_API_GET_LIMIT } from '../../constants';

export const makePositiveIntegerQueryParser = (title: string) =>
  z
    .number()
    .int()
    .or(z.string().regex(/^\d+$/).transform(Number))
    .refine((n) => n >= 0, {
      message: `${title} must be a positive integer`,
    });

export const queryLimitParser = makePositiveIntegerQueryParser('Limit');

// for 'get' query params, .or shenanigans
export const firestoreQueryLimitParser = queryLimitParser.refine((n) => n <= FIRESTORE_API_GET_LIMIT, {
  message: `limit must be less than ${FIRESTORE_API_GET_LIMIT}`,
});

export const queryOffsetParser = makePositiveIntegerQueryParser('Offset');

// for 'get' query params, .or shenanigans
export const firestoreQueryOffsetParser = queryOffsetParser; // no special additions

export const makeLimitParser = <T extends z.ZodType>(parser: T) =>
  z.object({
    limit: parser,
  });

export const makeOffsetParser = <T extends z.ZodType>(parser: T) =>
  z.object({
    offset: parser,
  });

export const makeLimitOffsetParser = <LP extends z.ZodType, OP extends z.ZodType>(limitParser: LP, offsetParser: OP) =>
  makeLimitParser(offsetParser).merge(makeOffsetParser(limitParser));

export const queryOffsetLimitParser = makeLimitOffsetParser(queryOffsetParser.default(0), queryLimitParser.optional());

const firestoreListLimitParser = firestoreQueryLimitParser.optional().default(FIRESTORE_API_GET_LIMIT);
const firestoreListOffsetParser = firestoreQueryOffsetParser.optional().default(0);
export const firestoreListLimitParamsParser = makeLimitParser(firestoreListLimitParser);
export const firestoreListOffsetParamsParser = makeOffsetParser(firestoreListOffsetParser);

// TODO test that limit can't leak over 1000 or to -1
export const firestoreListParamsParser = makeLimitOffsetParser(firestoreListOffsetParser, firestoreListLimitParser);
