"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.routingStrategyQueryParser = exports.routingStrategyParser = exports.routingStrategyThresholdQueryParser = exports.routingStrategyActivationThresholdParser = exports.routingStrategyThresholdParser = exports.routingStrategyThresholdQueryParamsParser = exports.routingStrategyFanoutQueryParser = exports.routingStrategyFanoutParser = exports.routingStrategyKeyParser = exports.routingStrategyActivationThresholdKeyParser = exports.routingStrategyThresholdKeyParser = exports.routingStrategyFanoutKeyParser = void 0;
const zod_1 = __importDefault(require("zod"));
const constants_1 = require("./constants");
const queryParams_1 = require("../../common/parsers/queryParams");
const defaults_1 = require("./defaults");
exports.routingStrategyFanoutKeyParser = zod_1.default.literal(constants_1.ROUTING_STRATEGY_KEY_FANOUT);
exports.routingStrategyThresholdKeyParser = zod_1.default.literal(constants_1.ROUTING_STRATEGY_KEY_THRESHOLD);
exports.routingStrategyActivationThresholdKeyParser = zod_1.default.literal(constants_1.ROUTING_STRATEGY_KEY_ACTIVATION_THRESHOLD);
exports.routingStrategyKeyParser = exports.routingStrategyFanoutKeyParser
    .or(exports.routingStrategyThresholdKeyParser)
    .or(exports.routingStrategyActivationThresholdKeyParser);
exports.routingStrategyFanoutParser = zod_1.default.object({
    key: exports.routingStrategyFanoutKeyParser,
});
exports.routingStrategyFanoutQueryParser = zod_1.default.object({
    key: exports.routingStrategyFanoutKeyParser,
});
exports.routingStrategyThresholdQueryParamsParser = zod_1.default.object({
    threshold: (0, queryParams_1.makePositiveIntegerQueryParser)('Threshold'),
});
exports.routingStrategyThresholdParser = zod_1.default.object({
    key: exports.routingStrategyThresholdKeyParser,
    params: zod_1.default.object({
        threshold: zod_1.default.number(),
    }),
});
exports.routingStrategyActivationThresholdParser = zod_1.default.object({
    key: exports.routingStrategyActivationThresholdKeyParser,
    // activation value depends on the property on the node
    // params: z.object({
    //   threshold: z.number(),
    // })
});
exports.routingStrategyThresholdQueryParser = zod_1.default.object({
    key: exports.routingStrategyThresholdKeyParser,
    params: exports.routingStrategyThresholdQueryParamsParser,
});
exports.routingStrategyParser = zod_1.default.discriminatedUnion('key', [
    exports.routingStrategyFanoutParser,
    exports.routingStrategyThresholdParser,
    exports.routingStrategyActivationThresholdParser,
]);
// doesn't work well with a flat object
// export const routingStrategyQueryParser = z.discriminatedUnion('strategy', [
//   routingStrategyFanoutQueryParser,
//   routingStrategyThresholdQueryParser,
// ]);
// nested
exports.routingStrategyQueryParser = zod_1.default.object({
    strategy: exports.routingStrategyParser.default(defaults_1.DEFAULT_ROUTING_STRATEGY),
});
