import { Isomorphism } from '../utils/types';
import {
  ApiPayinDstComposite,
  ApiPayinSrcComposite,
  PayinCompositeId,
  PayinCompositeIdSerialized,
  PayinDst,
  PayinSrc,
} from './types';
import {
  payinApiDstCompositeParser,
  payinApiSrcCompositeParser,
  payinCompositeIdParser,
  SEPARATOR,
} from './parsers.common';

export const payinApiSrcIsomorphism: Isomorphism<ApiPayinSrcComposite, PayinSrc> = {
  to: (x) => `${x.type}${SEPARATOR}${x.id}` as PayinSrc,
  from: (x) => payinApiSrcCompositeParser.parse(x),
};
export const payinApiDstIsomorphism: Isomorphism<ApiPayinDstComposite, PayinDst> = {
  to: (x) => `${x.type}${SEPARATOR}${x.id}` as PayinDst,
  from: (x) => payinApiDstCompositeParser.parse(x),
};

export const payinCompositeKeyIsomorphism: Isomorphism<PayinCompositeId, PayinCompositeIdSerialized> = {
  to: (x) => `${x.provider}${SEPARATOR}${x.originId}` as PayinCompositeIdSerialized,
  from: (x) => {
    const [provider, originId] = x.split(SEPARATOR) as [PayinCompositeId['provider'], PayinCompositeId['originId']];
    return payinCompositeIdParser.parse({ provider, originId });
  },
};
