import z, { ZodType } from 'zod';
import { userIdParser } from '../user/parsers';
import { payoutMethodOriginTypeParser } from '../payoutMethod/parsers';
import { currencyParser } from '../currencies/parsers';
import {
  payoutMethodOriginTypeLineGranularParser,
  payoutMethodOriginTypeNonLineGranularParser,
} from './parsers.granularity';
import { nonEmptyStringParser } from '../utils/parsers';
import { PayoutBatchKey } from './types';

const makeCompositeKeyParsers = <T extends string, P extends ZodType<T>>(originTypeParser: P) => ({
  userId: userIdParser,
  originType: originTypeParser,
  currency: currencyParser,
});

export const compositeKeyParsers = makeCompositeKeyParsers(payoutMethodOriginTypeParser);
export const compositeKeyLineGranularParsers = makeCompositeKeyParsers(payoutMethodOriginTypeLineGranularParser);
export const compositeKeyNonLineGranularParsers = makeCompositeKeyParsers(payoutMethodOriginTypeNonLineGranularParser);
export const payoutBatchItemCompositeKeyParser = z.object(compositeKeyParsers);
export const payoutBatchItemCompositeKeyLineGranularParser = z.object(compositeKeyLineGranularParsers);
export const payoutBatchItemCompositeKeyNonLineGranularParser = z.object(compositeKeyNonLineGranularParsers);
export const payoutBatchKeyParser = nonEmptyStringParser.transform((s) => s as PayoutBatchKey);
