import z, { ZodType } from 'zod';
import {
  payoutBatchItemCompositeKeyIsomorphism,
  payoutBatchItemCompositeKeyLineableIsomorphism,
  payoutBatchItemCompositeKeyNonLineableIsomorphism,
  paypalMassPayoutLineNoteFullRegexS,
} from './isomorphism';
import {
  compositeKeyParsers,
  payoutBatchItemCompositeKeyLineGranularParser,
  payoutBatchItemCompositeKeyNonLineGranularParser,
  payoutBatchItemCompositeKeyParser,
} from './parsers.common';
import type { Isomorphism } from '../utils/types';
import { assertStringStrict } from '../utils/string';

const makePayoutBatchItemCompositeKeySerializedParser = <T, A extends T, B extends string>(
  isomorphism: Isomorphism<A, B>,
  parser: ZodType<T>
) =>
  z
    .string()
    .min(
      Object.keys(compositeKeyParsers).length +
        (Object.keys(compositeKeyParsers).length - 1) /*at least 1 letter separator is assumed*/
    )
    .transform((s, ctx) => {
      try {
        return isomorphism.from(s as B);
      } catch (e) {
        // zod error
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        if ((e as any)?.issues?.length) {
          const ze = e as z.ZodError;
          ze.issues.forEach(ctx.addIssue.bind(ctx));
          return;
        }
        throw e;
      }
    })
    .superRefine((o, ctx) => {
      const r = parser.safeParse(o);
      if (!r.success) r.error.issues.forEach(ctx.addIssue.bind(ctx));
    })
    .transform((r) => {
      if (!r)
        throw new Error(
          "panic! empty transform argument shouldn't happen here as it was reported as an error in a previous call"
        );
      return isomorphism.to(r);
    });

export const payoutBatchItemCompositeKeySerializedParser = makePayoutBatchItemCompositeKeySerializedParser(
  payoutBatchItemCompositeKeyIsomorphism,
  payoutBatchItemCompositeKeyParser
);
export const payoutBatchLineableItemCompositeKeySerializedParser = makePayoutBatchItemCompositeKeySerializedParser(
  payoutBatchItemCompositeKeyLineableIsomorphism,
  payoutBatchItemCompositeKeyLineGranularParser
);
export const payoutBatchNonLineableItemCompositeKeySerializedParser = makePayoutBatchItemCompositeKeySerializedParser(
  payoutBatchItemCompositeKeyNonLineableIsomorphism,
  payoutBatchItemCompositeKeyNonLineGranularParser
);

export const payoutBatchItemCompositeKeyLineGranularSerializedParserSafeParse = (s: string) =>
  payoutBatchLineableItemCompositeKeySerializedParser.safeParse(assertStringStrict(s));
export const payoutBatchItemCompositeKeyNonLineGranularSerializedParserSafeParse = (s: string) =>
  payoutBatchNonLineableItemCompositeKeySerializedParser.safeParse(assertStringStrict(s));

export const massPaypalPayoutItemNoteParser = z.string().regex(RegExp(paypalMassPayoutLineNoteFullRegexS));
