import z from 'zod';
import type { ApiTip } from './types';
import { cast } from '../utils/types';
import type { ListResponseTrait } from '../response/types';
import {
  payinApiSrcDstCompositeParser,
  payinListQueryParser,
  payinAmountUnitParser,
  payinContextParser,
  payinSerializedKeyParser,
} from '../payin/parsers';
import { idempotencyKeyBodyParser } from '../idempotency/parser';
import { PAYIN_KEY_PARAM_NAME } from '../payin/constants';
import { payinOriginTypeParser } from '../payin/parsers.common';
import { booleanQueryParser } from '../parsers';

export const tipListQueryParser = payinListQueryParser;

export const tipListResponseParser = z.any(/*stub*/).transform(cast<ListResponseTrait<ApiTip>>());
// export const tipStatusParser = z.enum(TipStatus);
export const tipOriginTypeParser = payinOriginTypeParser;

const tipCreateCommonParser = z
  .object({})
  .merge(payinAmountUnitParser)
  .merge(payinContextParser)
  .merge(idempotencyKeyBodyParser);

// "api" tip shape; not for use to parse real stripe tips
export const tipCreateInputParser = z.object({}).merge(payinApiSrcDstCompositeParser).merge(tipCreateCommonParser);

export const tipCreateResponseParser = z.any(/*stub*/).transform(cast<ApiTip>());

export const tipParamsParser = z.object({
  [PAYIN_KEY_PARAM_NAME]: payinSerializedKeyParser,
});

export const tipFetchQueryParser = z.object({
  bump: booleanQueryParser,
});

// export const tipUpdateInputParser = tipApiAmountParser.partial();
// export const tipUpdateParamsParser = tipGetOneParamsParser;
// export const tipUpdateResponseParser = tipCreateResponseParser;
// export const tipDeleteParamsParser = tipGetOneParamsParser;
