import z from 'zod';
import { Subset } from '../utils/types';
import {
  tipCreateInputParser,
  tipCreateResponseParser,
  tipFetchQueryParser,
  tipListQueryParser,
  tipListResponseParser,
  tipParamsParser,
} from './parsers';
import { PAYIN_ORIGIN_TYPES, PAYIN_PROVIDERS } from '../payin/constants';
import { StripeTipId } from '../stripe/types';
import { Currency } from '../currencies/types';
import { ApiPayinResultMixin, PayinDst, PayinSrc, PayinSrcDstApiQueryMixin } from '../payin/types';
import { PayinProviderOriginType } from '../payinProvider/types';
import { payinApiDstCompositeParser, payinApiSrcCompositeParser } from '../payin/parsers.common';

export type TipId = StripeTipId;
export type TipProvider = typeof PAYIN_PROVIDERS[number];
export type TipOriginType = typeof PAYIN_ORIGIN_TYPES[number];
type TipOriginIdPerTypeMapping = {
  paypal: never; // PaypalTipId,
  stripe: StripeTipId;
};
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipOriginIdPerTypeMapping1 = Subset<keyof TipOriginIdPerTypeMapping, TipOriginType>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipOriginIdPerTypeMapping2 = Subset<TipOriginType, keyof TipOriginIdPerTypeMapping>;
export type TipOriginIdForType<T extends TipOriginType> = TipOriginIdPerTypeMapping[T];

// currently mirrors Stripe https://stripe.com/docs/payments/intents a lot; TODO inner statuses
const TIP_STATUSES = [
  'INTENT',
  'PROCESSING',
  'CANCELED',
  'SUCCEEDED',
  'REQUIRES_ACTION',
  'REQUIRES_CAPTURE',
  'REQUIRES_CONFIRMATION',
  'REQUIRES_PAYMENT_METHOD',
] as const;

export type TipStatus = typeof TIP_STATUSES[number];

export type ApiTip<P extends PayinProviderOriginType = PayinProviderOriginType> = {
  amt: number;
  unit: Currency;
  src: PayinSrc;
  dst: PayinDst;
  created: number;
} & ApiPayinResultMixin<'tip', P>;

export type ApiTipSrcComposite = z.infer<typeof payinApiSrcCompositeParser>;
export type ApiTipDstComposite = z.infer<typeof payinApiDstCompositeParser>;

export type TipFetchParams = z.input<typeof tipParamsParser>;
export type TipFetchParamsTransformed = z.output<typeof tipParamsParser>;
export type TipFetchQuery = z.input<typeof tipFetchQueryParser>;
export type TipFetchQueryTransformed = z.output<typeof tipFetchQueryParser>;
export type TipListApiQuery = z.input<typeof tipListQueryParser>;
export type TipListApiQueryTransformed = z.output<typeof tipListQueryParser>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertTipListApiQuery = Subset<TipListApiQueryTransformed, PayinSrcDstApiQueryMixin>;
export type TipListApiResponse = z.infer<typeof tipListResponseParser>; // questionable whether we want to encode channel type there; probably not
// export type TipUpdateApiParams = z.infer<typeof tipUpdateParamsParser>;
// export type TipUpdateApiBody = z.infer<typeof tipUpdateInputParser>;
// export type TipUpdateApiResponse = z.infer<typeof tipUpdateResponseParser>;
export type TipCreateApiBody = z.input<typeof tipCreateInputParser>;
export type TipCreateApiBodyTransformed = z.output<typeof tipCreateInputParser>;
export type TipCreateApiResponse = z.infer<typeof tipCreateResponseParser>;
// export type TipDeleteApiParams = z.infer<typeof tipDeleteParamsParser>;
